@@ -34,6 +34,18 @@ class InstallData():
         self.install_scripts = []
         self.install_subdirs = []
 
+class ExecutableSerialisation():
+    def __init__(self, name, fname, cmd_args, env, is_cross, exe_wrapper,
+                 workdir, extra_paths):
+        self.name = name
+        self.fname = fname
+        self.cmd_args = cmd_args
+        self.env = env
+        self.is_cross = is_cross
+        self.exe_runner = exe_wrapper
+        self.workdir = workdir
+        self.extra_paths = extra_paths
+
 class TestSerialisation:
     def __init__(self, name, suite, fname, is_cross, exe_wrapper, is_parallel, cmd_args, env,
                  should_fail, valgrind_args, timeout, workdir, extra_paths):
@@ -154,6 +166,35 @@ class Backend():
                 raise MesonException('Unknown data type in object list.')
         return obj_list
 
+    def serialise_executable(self, exe, cmd_args, workdir, env={}):
+        import uuid
+        # Can't just use exe.name here; it will likely be run more than once
+        scratch_file = 'meson_exe_{0}_{1}.dat'.format(exe.name,
+                                                      str(uuid.uuid4())[:8])
+        exe_data = os.path.join(self.environment.get_scratch_dir(), scratch_file)
+        with open(exe_data, 'wb') as f:
+            if isinstance(exe, dependencies.ExternalProgram):
+                exe_fullpath = exe.fullpath
+            else:
+                exe_fullpath = [os.path.join(self.environment.get_build_dir(),
+                                             self.get_target_filename(exe))]
+            is_cross = self.environment.is_cross_build() and \
+                self.environment.cross_info.need_cross_compiler() and \
+                self.environment.cross_info.need_exe_wrapper()
+            if is_cross:
+                exe_wrapper = self.environment.cross_info.config['binaries'].get('exe_wrapper', None)
+            else:
+                exe_wrapper = None
+            if mesonlib.is_windows():
+                extra_paths = self.determine_windows_extra_paths(exe)
+            else:
+                extra_paths = []
+            es = ExecutableSerialisation(exe.name, exe_fullpath, cmd_args, env,
+                                         is_cross, exe_wrapper, workdir,
+                                         extra_paths)
+            pickle.dump(es, f)
+        return exe_data
+
     def serialise_tests(self):
         test_data = os.path.join(self.environment.get_scratch_dir(), 'meson_test_setup.dat')
         datafile = open(test_data, 'wb')
@@ -163,6 +204,7 @@ class Backend():
         datafile = open(benchmark_data, 'wb')
         self.write_benchmark_file(datafile)
         datafile.close()
+        return (test_data, benchmark_data)
 
     def has_source_suffix(self, target, suffix):
         for s in target.get_sources():
@@ -244,6 +286,7 @@ class Backend():
 
     def generate_basic_compiler_args(self, target, compiler):
         commands = []
+        commands += self.get_cross_stdlib_args(target, compiler)
         commands += compiler.get_always_args()
         commands += compiler.get_warn_args(self.environment.coredata.get_builtin_option('warning_level'))
         commands += compiler.get_option_compile_args(self.environment.coredata.compiler_options)
@@ -315,7 +358,9 @@ class Backend():
                 fname = exe.fullpath
             else:
                 fname = [os.path.join(self.environment.get_build_dir(), self.get_target_filename(t.get_exe()))]
-            is_cross = self.environment.is_cross_build() and self.environment.cross_info.need_cross_compiler()
+            is_cross = self.environment.is_cross_build() and \
+                self.environment.cross_info.need_cross_compiler() and \
+                self.environment.cross_info.need_exe_wrapper()
             if is_cross:
                 exe_wrapper = self.environment.cross_info.config['binaries'].get('exe_wrapper', None)
             else:
@@ -366,8 +411,9 @@ class Backend():
 
     def exe_object_to_cmd_array(self, exe):
         if self.environment.is_cross_build() and \
-                isinstance(exe, build.BuildTarget) and exe.is_cross:
-            if 'exe_wrapper'  not in self.environment.cross_info:
+           self.environment.cross_info.need_exe_wrapper() and \
+           isinstance(exe, build.BuildTarget) and exe.is_cross:
+            if 'exe_wrapper' not in self.environment.cross_info.config:
                 s = 'Can not use target %s as a generator because it is cross-built\n'
                 s += 'and no exe wrapper is defined. You might want to set it to native instead.'
                 s = s % exe.name
@@ -387,6 +433,29 @@ class Backend():
                 final_args.append(a)
         return final_args
 
+    def replace_outputs(self, args, private_dir, output_list):
+        newargs = []
+        regex = re.compile('@OUTPUT(\d+)@')
+        for arg in args:
+            m = regex.search(arg)
+            while m is not None:
+                index = int(m.group(1))
+                src = '@OUTPUT%d@' % index
+                arg = arg.replace(src, os.path.join(private_dir, output_list[index]))
+                m = regex.search(arg)
+            newargs.append(arg)
+        return newargs
+
+    def get_custom_target_provided_libraries(self, target):
+        libs = []
+        for t in target.get_generated_sources():
+            if not isinstance(t, build.CustomTarget):
+                continue
+            for f in t.output:
+                if self.environment.is_library(f):
+                    libs.append(os.path.join(self.get_target_dir(t), f))
+        return libs
+
     def eval_custom_target_command(self, target, absolute_paths=False):
         if not absolute_paths:
             ofilenames = [os.path.join(self.get_target_dir(target), i) for i in target.output]
