@@ -176,9 +176,6 @@ int dummy;
         self.generate_phony(outfile)
         outfile.write('# Build rules for targets\n\n')
         [self.generate_target(t, outfile) for t in self.build.get_targets().values()]
-        if len(self.build.pot) > 0:
-            outfile.write('# Build rules for localisation.\n\n')
-            self.generate_po(outfile)
         outfile.write('# Test rules\n\n')
         self.generate_tests(outfile)
         outfile.write('# Install rules\n\n')
@@ -188,6 +185,7 @@ int dummy;
             outfile.write('# Coverage rules\n\n')
             self.generate_coverage_rules(outfile)
         outfile.write('# Suffix\n\n')
+        self.generate_utils(outfile)
         self.generate_ending(outfile)
         # Only ovewrite the old build file after the new one has been
         # fully created.
@@ -199,7 +197,10 @@ int dummy;
     def generate_compdb(self):
         ninja_exe = environment.detect_ninja()
         builddir = self.environment.get_build_dir()
-        jsondb = subprocess.check_output([ninja_exe, '-t', 'compdb', 'c_COMPILER', 'cpp_COMPILER'], cwd=builddir)
+        try:
+            jsondb = subprocess.check_output([ninja_exe, '-t', 'compdb', 'c_COMPILER', 'cpp_COMPILER'], cwd=builddir)
+        except Exception:
+                raise MesonException('Could not create compilation database.')
         open(os.path.join(builddir, 'compile_commands.json'), 'wb').write(jsondb)
 
     # Get all generated headers. Any source file might need them so
@@ -341,6 +342,7 @@ int dummy;
     def generate_custom_target(self, target, outfile):
         (srcs, ofilenames, cmd) = self.eval_custom_target_command(target)
         deps = []
+        desc = 'Generating {0} with a {1} command.'
         for i in target.get_dependencies():
             # FIXME, should not grab element at zero but rather expand all.
             if isinstance(i, list):
@@ -364,9 +366,23 @@ int dummy;
                 tmp = [tmp]
             for fname in tmp:
                 elem.add_dep(os.path.join(self.get_target_dir(d), fname))
+        # Windows doesn't have -rpath, so for EXEs that need DLLs built within
+        # the project, we need to set PATH so the DLLs are found. We use
+        # a serialized executable wrapper for that and check if the
+        # CustomTarget command needs extra paths first.
+        if mesonlib.is_windows() and \
+           self.determine_windows_extra_paths(target.command[0]):
+            exe_data = self.serialise_executable(target.command[0], cmd[1:],
+                # All targets are built from the build dir
+                self.environment.get_build_dir())
+            cmd = [sys.executable, self.environment.get_build_command(),
+                   '--internal', 'exe', exe_data]
+            cmd_type = 'meson_exe.py custom'
+        else:
+            cmd_type = 'custom'
 
         elem.add_item('COMMAND', cmd)
-        elem.add_item('description', 'Generating %s with a custom command.' % target.name)
+        elem.add_item('description',  desc.format(target.name, cmd_type))
         elem.write(outfile)
         self.processed_targets[target.name + target.type_suffix()] = True
 
@@ -394,9 +410,11 @@ int dummy;
         if isinstance(texe, build.Executable):
             abs_exe = os.path.join(self.environment.get_build_dir(), self.get_target_filename(texe))
             deps.append(self.get_target_filename(texe))
-            if self.environment.is_cross_build() \
-                and self.environment.cross_info.config['binaries'].get('exe_wrapper', None) is not None:
-                cmd += [self.environment.cross_info.config['binaries']['exe_wrapper']]
+            if self.environment.is_cross_build() and \
+               self.environment.cross_info.need_exe_wrapper():
+                exe_wrap = self.environment.cross_info.config['binaries'].get('exe_wrapper', None)
+                if exe_wrap is not None:
+                    cmd += [exe_wrap]
             cmd.append(abs_exe)
         else:
             cmd.append(target.command)
@@ -407,51 +425,33 @@ int dummy;
         elem.write(outfile)
         self.processed_targets[target.name + target.type_suffix()] = True
 
-    def generate_po(self, outfile):
-        for p in self.build.pot:
-            (packagename, languages, subdir) = p
-            input_file = os.path.join(subdir, 'POTFILES')
-            elem = NinjaBuildElement(self.all_outputs, 'pot', 'GEN_POT', [])
-            elem.add_item('PACKAGENAME', packagename)
-            elem.add_item('OUTFILE', packagename + '.pot')
-            elem.add_item('FILELIST', os.path.join(self.environment.get_source_dir(), input_file))
-            elem.add_item('OUTDIR', os.path.join(self.environment.get_source_dir(), subdir))
-            elem.write(outfile)
-            for l in languages:
-                infile = os.path.join(self.environment.get_source_dir(), subdir, l + '.po')
-                outfilename = os.path.join(subdir, l + '.gmo')
-                lelem = NinjaBuildElement(self.all_outputs, outfilename, 'GEN_GMO', infile)
-                lelem.add_item('INFILE', infile)
-                lelem.add_item('OUTFILE', outfilename)
-                lelem.write(outfile)
-
     def generate_coverage_rules(self, outfile):
         (gcovr_exe, lcov_exe, genhtml_exe) = environment.find_coverage_tools()
         added_rule = False
         if gcovr_exe:
             added_rule = True
             elem = NinjaBuildElement(self.all_outputs, 'coverage-xml', 'CUSTOM_COMMAND', '')
-            elem.add_item('COMMAND', [gcovr_exe, '-x', '-r', self.environment.get_build_dir(),\
+            elem.add_item('COMMAND', [gcovr_exe, '-x', '-r', self.environment.get_source_dir(),\
                                       '-o', os.path.join(self.environment.get_log_dir(), 'coverage.xml')])
             elem.add_item('DESC', 'Generating XML coverage report.')
             elem.write(outfile)
             elem = NinjaBuildElement(self.all_outputs, 'coverage-text', 'CUSTOM_COMMAND', '')
-            elem.add_item('COMMAND', [gcovr_exe, '-r', self.environment.get_build_dir(),\
+            elem.add_item('COMMAND', [gcovr_exe, '-r', self.environment.get_source_dir(),\
                                       '-o', os.path.join(self.environment.get_log_dir(), 'coverage.txt')])
             elem.add_item('DESC', 'Generating text coverage report.')
             elem.write(outfile)
         if lcov_exe and genhtml_exe:
             added_rule = True
-            phony_elem = NinjaBuildElement(self.all_outputs, 'coverage-html', 'phony', 'coveragereport/index.html')
-            phony_elem.write(outfile)
-
-            elem = NinjaBuildElement(self.all_outputs, 'coveragereport/index.html', 'CUSTOM_COMMAND', '')
             htmloutdir = os.path.join(self.environment.get_log_dir(), 'coveragereport')
+            covinfo = os.path.join(self.environment.get_log_dir(), 'coverage.info')
+            phony_elem = NinjaBuildElement(self.all_outputs, 'coverage-html', 'phony', os.path.join(htmloutdir, 'index.html'))
+            phony_elem.write(outfile)
+            elem = NinjaBuildElement(self.all_outputs, os.path.join(htmloutdir, 'index.html'), 'CUSTOM_COMMAND', '')
             command = [lcov_exe, '--directory', self.environment.get_build_dir(),\
-                       '--capture', '--output-file', 'coverage.info', '--no-checksum',\
+                       '--capture', '--output-file', covinfo, '--no-checksum',\
                        '&&', genhtml_exe, '--prefix', self.environment.get_build_dir(),\
                        '--output-directory', htmloutdir, '--title', 'Code coverage',\
-                       '--legend', '--show-details', 'coverage.info']
+                       '--legend', '--show-details', covinfo]
             elem.add_item('COMMAND', command)
             elem.add_item('DESC', 'Generating HTML coverage report.')
             elem.write(outfile)
@@ -459,8 +459,6 @@ int dummy;
             mlog.log(mlog.red('Warning:'), 'coverage requested but neither gcovr nor lcov/genhtml found.')
 
     def generate_install(self, outfile):
-        script_root = self.environment.get_script_dir()
-        install_script = os.path.join(script_root, 'meson_install.py')
         install_data_file = os.path.join(self.environment.get_scratch_dir(), 'install.dat')
         d = InstallData(self.environment.get_source_dir(),
                         self.environment.get_build_dir(),
@@ -475,7 +473,6 @@ int dummy;
         self.generate_header_install(d)
         self.generate_man_install(d)
         self.generate_data_install(d)
-        self.generate_po_install(d, elem)
         self.generate_custom_install_script(d)
         self.generate_subdir_install(d)
         elem.write(outfile)
@@ -483,17 +480,6 @@ int dummy;
         ofile = open(install_data_file, 'wb')
         pickle.dump(d, ofile)
 
-    def generate_po_install(self, d, elem):
-        for p in self.build.pot:
-            (package_name, languages, subdir) = p
-            # FIXME: assumes only one po package per source
-            d.po_package_name = package_name
-            for lang in languages:
-                rel_src =  os.path.join(subdir, lang + '.gmo')
-                src_file = os.path.join(self.environment.get_build_dir(), rel_src)
-                d.po.append((src_file, self.environment.coredata.get_builtin_option('localedir'), lang))
-                elem.add_dep(rel_src)
-
     def generate_target_install(self, d):
         libdir = self.environment.get_libdir()
         bindir = self.environment.get_bindir()
@@ -537,7 +523,7 @@ int dummy;
                 if subdir is None:
                     subdir = os.path.join(manroot, 'man' + num)
                 srcabs = os.path.join(self.environment.get_source_dir(), m.get_source_subdir(), f)
-                dstabs = os.path.join(subdir, f + '.gz')
+                dstabs = os.path.join(subdir, os.path.split(f)[1] + '.gz')
                 i = [srcabs, dstabs]
                 d.man.append(i)
 
@@ -559,9 +545,17 @@ int dummy;
 
     def generate_subdir_install(self, d):
         for sd in self.build.get_install_subdirs():
-            src_dir = os.path.join(self.environment.get_source_dir(), sd.source_subdir, sd.installable_subdir)
+            inst_subdir = sd.installable_subdir.rstrip('/')
+            idir_parts = inst_subdir.split('/')
+            if len(idir_parts) > 1:
+                subdir = os.path.join(sd.source_subdir, '/'.join(idir_parts[:-1]))
+                inst_dir = idir_parts[-1]
+            else:
+                subdir = sd.source_subdir
+                inst_dir = sd.installable_subdir
+            src_dir = os.path.join(self.environment.get_source_dir(), subdir)
             dst_dir = os.path.join(self.environment.get_prefix(), sd.install_dir)
-            d.install_subdirs.append([src_dir, dst_dir])
+            d.install_subdirs.append([src_dir, inst_dir, dst_dir])
 
     def write_test_suite_targets(self, cmd, outfile):
         suites = {}
@@ -582,10 +576,9 @@ int dummy;
             elem.write(outfile)
 
     def generate_tests(self, outfile):
-        self.serialise_tests()
+        (test_data, benchmark_data) = self.serialise_tests()
         valgrind = environment.find_valgrind()
         script_root = self.environment.get_script_dir()
-        test_data = os.path.join(self.environment.get_scratch_dir(), 'meson_test_setup.dat')
         cmd = [ sys.executable, self.environment.get_build_command(), '--internal', 'test' ]
         if not self.environment.coredata.get_builtin_option('stdsplit'):
             cmd += ['--no-stdsplit']
@@ -608,7 +601,6 @@ int dummy;
 
         # And then benchmarks.
         benchmark_script = os.path.join(script_root, 'meson_benchmark.py')
-        benchmark_data = os.path.join(self.environment.get_scratch_dir(), 'meson_benchmark_setup.dat')
         cmd = [sys.executable, self.environment.get_build_command(), '--internal', 'benchmark', benchmark_data]
         elem = NinjaBuildElement(self.all_outputs, 'benchmark', 'CUSTOM_COMMAND', ['all', 'PHONY'])
         elem.add_item('COMMAND', cmd)
@@ -639,25 +631,6 @@ int dummy;
         outfile.write(" command = %s %s %s %s %s %s --backend ninja\n" % c)
         outfile.write(' description = Regenerating build files\n')
         outfile.write(' generator = 1\n\n')
-        if len(self.build.pot) > 0:
-            self.generate_gettext_rules(outfile)
-        outfile.write('\n')
-
-    def generate_gettext_rules(self, outfile):
-        rule = 'rule GEN_POT\n'
-        command = " command = xgettext --package-name=$PACKAGENAME -p $OUTDIR -f $FILELIST -D '%s' -k_ -o $OUTFILE\n" % \
-        self.environment.get_source_dir()
-        desc = " description = Creating pot file for package $PACKAGENAME.\n"
-        outfile.write(rule)
-        outfile.write(command)
-        outfile.write(desc)
-        outfile.write('\n')
-        rule = 'rule GEN_GMO\n'
-        command = ' command = msgfmt $INFILE -o $OUTFILE\n'
-        desc = ' description = Generating gmo file $OUTFILE\n'
-        outfile.write(rule)
-        outfile.write(command)
-        outfile.write(desc)
         outfile.write('\n')
 
     def generate_phony(self, outfile):
@@ -1319,19 +1292,6 @@ rule FORTRAN_DEP_HACK
                 self.generate_pch_rule_for(langname, compiler, qstr, True, outfile)
         outfile.write('\n')
 
-    def replace_outputs(self, args, private_dir, output_list):
-        newargs = []
-        regex = re.compile('@OUTPUT(\d+)@')
-        for arg in args:
-            m = regex.search(arg)
-            while m is not None:
-                index = int(m.group(1))
-                src = '@OUTPUT%d@' % index
-                arg = arg.replace(src, os.path.join(private_dir, output_list[index]))
-                m = regex.search(arg)
-            newargs.append(arg)
-        return newargs
-
     def generate_custom_generator_rules(self, target, outfile):
         for genlist in target.get_generated_sources():
             if isinstance(genlist, build.CustomTarget):
@@ -1427,6 +1387,13 @@ rule FORTRAN_DEP_HACK
                 mod_files.append(os.path.join(dirname, mod_name))
         return mod_files
 
+    def get_cross_stdlib_args(self, target, compiler):
+        if not target.is_cross:
+            return []
+        if not self.environment.cross_info.has_stdlib(compiler.language):
+            return []
+        return compiler.get_no_stdinc_args()
+
     def generate_single_compile(self, target, outfile, src, is_generated=False, header_deps=[], order_deps=[]):
         if(isinstance(src, str) and src.endswith('.h')):
             raise RuntimeError('Fug')
@@ -1632,6 +1599,13 @@ rule FORTRAN_DEP_HACK
             elem.add_item('CROSS', '--cross-host=' + self.environment.cross_info.config['host_machine']['system'])
         elem.write(outfile)
 
+    def get_cross_stdlib_link_args(self, target, linker):
+        if isinstance(target, build.StaticLibrary) or not target.is_cross:
+            return []
+        if not self.environment.cross_info.has_stdlib(linker.language):
+            return []
+        return linker.get_no_stdlib_link_args()
+
     def generate_link(self, target, outfile, outname, obj_list, linker, extra_args=[]):
         if isinstance(target, build.StaticLibrary):
             linker_base = 'STATIC'
@@ -1645,6 +1619,9 @@ rule FORTRAN_DEP_HACK
         linker_rule = linker_base + crstr + '_LINKER'
         abspath = os.path.join(self.environment.get_build_dir(), target.subdir)
         commands = []
+        if not isinstance(target, build.StaticLibrary):
+            commands += self.build.get_global_link_args(linker)
+        commands += self.get_cross_stdlib_link_args(target, linker)
         commands += linker.get_linker_always_args()
         if not isinstance(target, build.StaticLibrary):
             commands += compilers.get_base_link_args(self.environment.coredata.base_options,
@@ -1704,16 +1681,6 @@ rule FORTRAN_DEP_HACK
         elem.add_item('LINK_ARGS', commands)
         return elem
 
-    def get_custom_target_provided_libraries(self, target):
-        libs = []
-        for t in target.get_generated_sources():
-            if not isinstance(t, build.CustomTarget):
-                continue
-            for f in t.output:
-                if self.environment.is_library(f):
-                    libs.append(os.path.join(self.get_target_dir(t), f))
-        return libs
-
     def determine_rpath_dirs(self, target):
         link_deps = target.get_all_link_deps()
         result = []
@@ -1789,6 +1756,16 @@ rule FORTRAN_DEP_HACK
                     other_deps.append(outfilename)
         return (src_deps, other_deps)
 
+    # For things like scan-build and other helper tools we might have.
+    def generate_utils(self, outfile):
+        cmd = [sys.executable, self.environment.get_build_command(),
+               '--internal', 'scanbuild', self.environment.source_dir, self.environment.build_dir,
+               sys.executable, self.environment.get_build_command()]
+        elem = NinjaBuildElement(self.all_outputs, 'scan-build', 'CUSTOM_COMMAND', 'PHONY')
+        elem.add_item('COMMAND', cmd)
+        elem.add_item('pool', 'console')
+        elem.write(outfile)
+
     def generate_ending(self, outfile):
         targetlist = [self.get_target_filename(t) for t in self.build.get_targets().values()\
                       if not isinstance(t, build.RunTarget)]
