@@ -39,6 +39,7 @@ class Vs2010Backend(backends.Backend):
         super().__init__(build)
         self.project_file_version = '10.0.30319.1'
         self.sources_conflicts = {}
+        self.platform_toolset = None
 
     def object_filename_from_source(self, target, source):
         basename = os.path.basename(source.fname)
@@ -69,13 +70,18 @@ class Vs2010Backend(backends.Backend):
                                                        if len(src_conflicts) > 1}
 
     def generate_custom_generator_commands(self, target, parent_node):
-        all_output_files = []
+        generator_output_files = []
         commands = []
         inputs = []
         outputs = []
+        custom_target_include_dirs = []
+        custom_target_output_files = []
         for genlist in target.get_generated_sources():
             if isinstance(genlist, build.CustomTarget):
-                all_output_files += [os.path.join(self.get_target_dir(genlist), i) for i in genlist.output]
+                custom_target_output_files += [os.path.join(self.get_target_dir(genlist), i) for i in genlist.output]
+                idir = self.relpath(self.get_target_dir(genlist), self.get_target_dir(target))
+                if idir not in custom_target_include_dirs:
+                    custom_target_include_dirs.append(idir)
             else:
                 generator = genlist.get_generator()
                 exe = generator.get_exe()
@@ -91,11 +97,12 @@ class Vs2010Backend(backends.Backend):
                         sole_output = ''
                     curfile = infilelist[i]
                     infilename = os.path.join(self.environment.get_source_dir(), curfile)
-                    outfiles = genlist.get_outputs_for(curfile)
-                    outfiles = [os.path.join(target_private_dir, of) for of in outfiles]
-                    all_output_files += outfiles
+                    outfiles_rel = genlist.get_outputs_for(curfile)
+                    outfiles = [os.path.join(target_private_dir, of) for of in outfiles_rel]
+                    generator_output_files += outfiles
                     args = [x.replace("@INPUT@", infilename).replace('@OUTPUT@', sole_output)\
                             for x in base_args]
+                    args = self.replace_outputs(args, target_private_dir, outfiles_rel)
                     args = [x.replace("@SOURCE_DIR@", self.environment.get_source_dir()).replace("@BUILD_DIR@", target_private_dir)
                             for x in args]
                     fullcmd = exe_arr + self.replace_extra_args(args, genlist)
@@ -111,7 +118,7 @@ class Vs2010Backend(backends.Backend):
             ET.SubElement(cbs, 'Message').text = 'Generating custom sources.'
             pg = ET.SubElement(parent_node, 'PropertyGroup')
             ET.SubElement(pg, 'CustomBuildBeforeTargets').text = 'ClCompile'
-        return all_output_files
+        return generator_output_files, custom_target_output_files, custom_target_include_dirs
 
     def generate(self, interp):
         self.resolve_source_conflicts()
@@ -260,6 +267,8 @@ class Vs2010Backend(backends.Backend):
                 lang = self.lang_from_source_file(i)
                 if lang not in languages:
                     languages.append(lang)
+            elif self.environment.is_library(i):
+                pass
             else:
                 # Everything that is not an object or source file is considered a header.
                 headers.append(i)
@@ -301,6 +310,8 @@ class Vs2010Backend(backends.Backend):
         ET.SubElement(type_config, 'ConfigurationType')
         ET.SubElement(type_config, 'CharacterSet').text = 'MultiByte'
         ET.SubElement(type_config, 'UseOfMfc').text = 'false'
+        if self.platform_toolset:
+            ET.SubElement(type_config, 'PlatformToolset').text = self.platform_toolset
         ET.SubElement(root, 'Import', Project='$(VCTargetsPath)\Microsoft.Cpp.props')
         direlem = ET.SubElement(root, 'PropertyGroup')
         fver = ET.SubElement(direlem, '_ProjectFileVersion')
@@ -377,6 +388,18 @@ class Vs2010Backend(backends.Backend):
         lang = Vs2010Backend.lang_from_source_file(source_file)
         ET.SubElement(parent_node, "AdditionalOptions").text = ' '.join(extra_args[lang]) + ' %(AdditionalOptions)'
 
+    @staticmethod
+    def has_objects(objects, additional_objects, generated_objects):
+        # Ignore generated objects, those are automatically used by MSBuild for VS2010, because they are part of
+        # the CustomBuildStep Outputs.
+        return len(objects) + len(additional_objects) > 0
+
+    @staticmethod
+    def add_generated_objects(node, generated_objects):
+        # Do not add generated objects to project file. Those are automatically used by MSBuild for VS2010, because
+        # they are part of the CustomBuildStep Outputs.
+        return
+
     @classmethod
     def quote_define_cmdline(cls, arg):
         return re.sub(r'^([-/])D(.*?)="(.*)"$', r'\1D\2=\"\3\"', arg)
@@ -433,11 +456,17 @@ class Vs2010Backend(backends.Backend):
         type_config = ET.SubElement(root, 'PropertyGroup', Label='Configuration')
         ET.SubElement(type_config, 'ConfigurationType').text = conftype
         ET.SubElement(type_config, 'CharacterSet').text = 'MultiByte'
+        if self.platform_toolset:
+            ET.SubElement(type_config, 'PlatformToolset').text = self.platform_toolset
         ET.SubElement(type_config, 'WholeProgramOptimization').text = 'false'
         ET.SubElement(type_config, 'UseDebugLibraries').text = 'true'
         ET.SubElement(root, 'Import', Project='$(VCTargetsPath)\Microsoft.Cpp.props')
-        generated_files = self.generate_custom_generator_commands(target, root)
+        generated_files, custom_target_output_files, generated_files_include_dirs = self.generate_custom_generator_commands(target, root)
         (gen_src, gen_hdrs, gen_objs, gen_langs) = self.split_sources(generated_files)
+        (custom_src, custom_hdrs, custom_objs, custom_langs) = self.split_sources(custom_target_output_files)
+        gen_src += custom_src
+        gen_hdrs += custom_hdrs
+        gen_langs += custom_langs
         direlem = ET.SubElement(root, 'PropertyGroup')
         fver = ET.SubElement(direlem, '_ProjectFileVersion')
         fver.text = self.project_file_version
@@ -455,7 +484,7 @@ class Vs2010Backend(backends.Backend):
         opt = ET.SubElement(clconf, 'Optimization')
         opt.text = 'disabled'
         inc_dirs = ['.', self.relpath(self.get_target_private_dir(target), self.get_target_dir(target)),
-                    proj_to_src_dir]
+                    proj_to_src_dir] + generated_files_include_dirs
 
         extra_args = {'c': [], 'cpp': []}
         for l, args in self.environment.coredata.external_args.items():
@@ -561,10 +590,14 @@ class Vs2010Backend(backends.Backend):
             rel_path = self.relpath(lobj.subdir, target.subdir)
             linkname = os.path.join(rel_path, lobj.get_import_filename())
             additional_links.append(linkname)
+        for lib in self.get_custom_target_provided_libraries(target):
+            additional_links.append(self.relpath(lib, self.get_target_dir(target)))
         additional_objects = []
         for o in self.flatten_object_list(target, down):
             assert(isinstance(o, str))
             additional_objects.append(o)
+        for o in custom_objs:
+            additional_objects.append(self.relpath(o, self.get_target_dir(target)))
         if len(additional_links) > 0:
             additional_links.append('%(AdditionalDependencies)')
             ET.SubElement(link, 'AdditionalDependencies').text = ';'.join(additional_links)
@@ -625,15 +658,15 @@ class Vs2010Backend(backends.Backend):
                 pch_file.text = os.path.split(header)[1]
                 self.add_additional_options(impl, inc_cl, extra_args, additional_options_set)
 
-        if len(objects) + len(additional_objects) > 0:
-            # Do not add gen_objs to project file. Those are automatically used by MSBuild, because they are part of
-            # the CustomBuildStep Outputs.
+        if self.has_objects(objects, additional_objects, gen_objs):
             inc_objs = ET.SubElement(root, 'ItemGroup')
             for s in objects:
                 relpath = s.rel_to_builddir(proj_to_src_root)
                 ET.SubElement(inc_objs, 'Object', Include=relpath)
             for s in additional_objects:
                 ET.SubElement(inc_objs, 'Object', Include=s)
+            self.add_generated_objects(inc_objs, gen_objs)
+
         ET.SubElement(root, 'Import', Project='$(VCTargetsPath)\Microsoft.Cpp.targets')
         # Reference the regen target.
         ig = ET.SubElement(root, 'ItemGroup')
@@ -675,6 +708,8 @@ class Vs2010Backend(backends.Backend):
         ET.SubElement(type_config, 'ConfigurationType').text = "Utility"
         ET.SubElement(type_config, 'CharacterSet').text = 'MultiByte'
         ET.SubElement(type_config, 'UseOfMfc').text = 'false'
+        if self.platform_toolset:
+            ET.SubElement(type_config, 'PlatformToolset').text = self.platform_toolset
         ET.SubElement(root, 'Import', Project='$(VCTargetsPath)\Microsoft.Cpp.props')
         direlem = ET.SubElement(root, 'PropertyGroup')
         fver = ET.SubElement(direlem, '_ProjectFileVersion')
@@ -752,6 +787,8 @@ if %%errorlevel%% neq 0 goto :VCEnd'''
         ET.SubElement(type_config, 'ConfigurationType')
         ET.SubElement(type_config, 'CharacterSet').text = 'MultiByte'
         ET.SubElement(type_config, 'UseOfMfc').text = 'false'
+        if self.platform_toolset:
+            ET.SubElement(type_config, 'PlatformToolset').text = self.platform_toolset
         ET.SubElement(root, 'Import', Project='$(VCTargetsPath)\Microsoft.Cpp.props')
         direlem = ET.SubElement(root, 'PropertyGroup')
         fver = ET.SubElement(direlem, '_ProjectFileVersion')
@@ -773,7 +810,6 @@ if %%errorlevel%% neq 0 goto :VCEnd'''
         ET.SubElement(midl, 'ProxyFileName').text = '%(Filename)_p.c'
         postbuild = ET.SubElement(action, 'PostBuildEvent')
         ET.SubElement(postbuild, 'Message')
-        test_data = os.path.join(self.environment.get_scratch_dir(), 'meson_test_setup.dat')
         test_command = [sys.executable,
                         self.environment.get_build_command(),
                         '--internal',
@@ -787,14 +823,12 @@ endlocal & call :cmErrorLevel %%errorlevel%% & goto :cmDone
 exit /b %%1
 :cmDone
 if %%errorlevel%% neq 0 goto :VCEnd'''
+        test_data = self.serialise_tests()[0]
         ET.SubElement(postbuild, 'Command').text =\
             cmd_templ % ('" "'.join(test_command), test_data)
         ET.SubElement(root, 'Import', Project='$(VCTargetsPath)\Microsoft.Cpp.targets')
         tree = ET.ElementTree(root)
         tree.write(ofname, encoding='utf-8', xml_declaration=True)
-        datafile = open(test_data, 'wb')
-        self.serialise_tests()
-        datafile.close()
         # ElementTree can not do prettyprinting so do it manually
         #doc = xml.dom.minidom.parse(ofname)
         #open(ofname, 'w').write(doc.toprettyxml())
