@@ -88,6 +88,7 @@ class Build:
         self.compilers = []
         self.cross_compilers = []
         self.global_args = {}
+        self.global_link_args = {}
         self.tests = []
         self.benchmarks = []
         self.headers = []
@@ -95,13 +96,13 @@ class Build:
         self.data = []
         self.static_linker = None
         self.static_cross_linker = None
-        self.pot = []
         self.subprojects = {}
         self.install_scripts = []
         self.postconf_scripts = []
         self.install_dirs = []
         self.dep_manifest_name = None
         self.dep_manifest = {}
+        self.cross_stdlibs = {}
 
     def has_language(self, language):
         for i in self.compilers:
@@ -151,6 +152,9 @@ class Build:
     def get_global_args(self, compiler):
         return self.global_args.get(compiler.get_language(), [])
 
+    def get_global_link_args(self, compiler):
+        return self.global_link_args.get(compiler.get_language(), [])
+
 class IncludeDirs():
     def __init__(self, curdir, dirs, is_system, extra_build_dirs=None):
         self.curdir = curdir
@@ -493,7 +497,8 @@ class BuildTarget():
                 for l in dep.libraries:
                     self.link(l)
                 # Those parts that are external.
-                extpart = dependencies.InternalDependency([],
+                extpart = dependencies.InternalDependency('undefined',
+                                                          [],
                                                           dep.compile_args,
                                                           dep.link_args,
                                                           [], [], [])
@@ -871,11 +876,12 @@ class CustomTarget:
             self.install = kwargs['install']
             if not isinstance(self.install, bool):
                 raise InvalidArguments('"install" must be boolean.')
-            if 'install_dir' not in kwargs:
-                raise InvalidArguments('"install_dir" not specified.')
-            self.install_dir = kwargs['install_dir']
-            if not(isinstance(self.install_dir, str)):
-                raise InvalidArguments('"install_dir" must be a string.')
+            if self.install:
+                if 'install_dir' not in kwargs:
+                    raise InvalidArguments('"install_dir" not specified.')
+                self.install_dir = kwargs['install_dir']
+                if not(isinstance(self.install_dir, str)):
+                    raise InvalidArguments('"install_dir" must be a string.')
         else:
             self.install = False
         self.build_always = kwargs.get('build_always', False)
