@@ -116,7 +116,7 @@ msvc_winlibs = ['kernel32.lib', 'user32.lib', 'gdi32.lib',
 
 
 base_options = {
-                'b_pch': coredata.UserBooleanOption('b_pch', 'Use precompiled headers', False),
+                'b_pch': coredata.UserBooleanOption('b_pch', 'Use precompiled headers', True),
                 'b_lto': coredata.UserBooleanOption('b_lto', 'Use link time optimization', False),
                 'b_sanitize': coredata.UserComboOption('b_sanitize',
                                                        'Code sanitizer to use',
@@ -311,6 +311,12 @@ class CCompiler(Compiler):
     def get_always_args(self):
         return []
 
+    def get_no_stdinc_args(self):
+        return ['-nostdinc']
+
+    def get_no_stdlib_link_args(self):
+        return ['-nostdlib']
+
     def get_warn_args(self, level):
         return self.warn_args[level]
 
@@ -414,30 +420,32 @@ class CCompiler(Compiler):
     def get_linker_search_args(self, dirname):
         return ['-L'+dirname]
 
-    def sanity_check(self, work_dir):
-        mlog.debug('Sanity testing C compiler:', ' '.join(self.exelist))
+    def sanity_check_impl(self, work_dir, sname, code):
+        mlog.debug('Sanity testing ' + self.language + ' compiler:', ' '.join(self.exelist))
         mlog.debug('Is cross compiler: %s.' % str(self.is_cross))
 
-        source_name = os.path.join(work_dir, 'sanitycheckc.c')
+        extra_flags = []
+        source_name = os.path.join(work_dir, sname)
+        binname = sname.rsplit('.', 1)[0]
         if self.is_cross:
-            binname = 'sanitycheckc_cross'
-        else:
-            binname = 'sanitycheckc'
+            binname += '_cross'
+            if self.exe_wrapper is None:
+                # Linking cross built apps is painful. You can't really
+                # tell if you should use -nostdlib or not and for example
+                # on OSX the compiler binary is the same but you need
+                # a ton of compiler flags to differentiate between
+                # arm and x86_64. So just compile.
+                extra_flags = self.get_compile_only_args()
+        # Is a valid executable output for all toolchains and platforms
+        binname += '.exe'
+        # Write binary check source
         binary_name = os.path.join(work_dir, binname)
         ofile = open(source_name, 'w')
-        ofile.write('int main(int argc, char **argv) { int class=0; return class; }\n')
+        ofile.write(code)
         ofile.close()
-        if self.is_cross and self.exe_wrapper is None:
-            # Linking cross built apps is painful. You can't really
-            # tell if you should use -nostdlib or not and for example
-            # on OSX the compiler binary is the same but you need
-            # a ton of compiler flags to differentiate between
-            # arm and x86_64. So just compile.
-            extra_flags = ['-c']
-        else:
-            extra_flags = []
-        cmdlist = self.exelist + extra_flags + [source_name, '-o', binary_name]
-        pc = subprocess.Popen(cmdlist, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        # Compile sanity check
+        cmdlist = self.exelist + extra_flags + [source_name] + self.get_output_args(binary_name)
+        pc = subprocess.Popen(cmdlist, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=work_dir)
         (stdo, stde) = pc.communicate()
         stdo = stdo.decode()
         stde = stde.decode()
@@ -448,7 +456,8 @@ class CCompiler(Compiler):
         mlog.debug(stde)
         mlog.debug('-----')
         if pc.returncode != 0:
-            raise EnvironmentException('Compiler %s can not compile programs.' % self.name_string())
+            raise EnvironmentException('Compiler {0} can not compile programs.'.format(self.name_string()))
+        # Run sanity check
         if self.is_cross:
             if self.exe_wrapper is None:
                 # Can't check if the binaries run so we have to assume they do
@@ -460,7 +469,11 @@ class CCompiler(Compiler):
         pe = subprocess.Popen(cmdlist)
         pe.wait()
         if pe.returncode != 0:
-            raise EnvironmentException('Executables created by C compiler %s are not runnable.' % self.name_string())
+            raise EnvironmentException('Executables created by {0} compiler {1} are not runnable.'.format(self.language, self.name_string()))
+
+    def sanity_check(self, work_dir):
+        code = 'int main(int argc, char **argv) { int class=0; return class; }\n'
+        return self.sanity_check_impl(work_dir, 'sanitycheckc.c', code)
 
     def has_header(self, hname, extra_args=[]):
         templ = '''#include<%s>
@@ -520,7 +533,13 @@ int main () {{ {1}; }}'''
         ofile = open(srcname, 'w')
         ofile.write(code)
         ofile.close()
-        extra_args = extra_args + self.get_output_args(dstname)
+        # Need to add buildtype args to select the CRT to use with MSVC
+        # This is needed especially while trying to link with static libraries
+        # since MSVC won't auto-select a CRT for us in that case and will error
+        # out asking us to select one.
+        extra_args = self.get_buildtype_args('debug') + \
+            self.unix_link_flags_to_native(extra_args) + \
+            self.get_output_args(dstname)
         p = self.compile(code, srcname, extra_args)
         try:
             os.remove(dstname)
@@ -539,7 +558,9 @@ int main () {{ {1}; }}'''
         ofile.close()
         exename = srcname + '.exe' # Is guaranteed to be executable on every platform.
         commands = self.get_exelist()
-        commands += extra_args
+        # Same reasoning as self.links() above
+        commands += self.get_buildtype_args('debug')
+        commands += self.unix_link_flags_to_native(extra_args)
         commands.append(srcname)
         commands += self.get_output_args(exename)
         p = subprocess.Popen(commands, cwd=os.path.split(srcname)[0], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
@@ -576,16 +597,30 @@ int main () {{ {1}; }}'''
         return RunResult(True, pe.returncode, so, se)
 
     def cross_sizeof(self, element, prefix, env, extra_args=[]):
-        templ = '''%s
+        element_exists_templ = '''#include <stdio.h>
+{0}
+int main(int argc, char **argv) {{
+    {1} something;
+}}
+'''
+        templ = '''#include <stdio.h>
+%s
 int temparray[%d-sizeof(%s)];
 '''
         try:
             extra_args += env.cross_info.config['properties'][self.language + '_args']
         except KeyError:
             pass
+        extra_args += self.get_no_optimization_args()
+        if not self.compiles(element_exists_templ.format(prefix, element)):
+            return -1
         for i in range(1, 1024):
             code = templ % (prefix, i, element)
             if self.compiles(code, extra_args):
+                if self.id == 'msvc':
+                    # MSVC refuses to construct an array of zero size, so
+                    # the test only succeeds when i is sizeof(element) + 1
+                    return i - 1
                 return i
         raise EnvironmentException('Cross checking sizeof overflowed.')
 
@@ -608,6 +643,11 @@ int main(int argc, char **argv) {
         return int(res.stdout)
 
     def cross_alignment(self, typename, env, extra_args=[]):
+        type_exists_templ = '''#include <stdio.h>
+int main(int argc, char **argv) {{
+    {0} something;
+}}
+'''
         templ = '''#include<stddef.h>
 struct tmp {
   char c;
@@ -620,9 +660,16 @@ int testarray[%d-offsetof(struct tmp, target)];
             extra_args += env.cross_info.config['properties'][self.language + '_args']
         except KeyError:
             pass
+        extra_args += self.get_no_optimization_args()
+        if not self.compiles(type_exists_templ.format(typename)):
+            return -1
         for i in range(1, 1024):
             code = templ % (typename, i)
             if self.compiles(code, extra_args):
+                if self.id == 'msvc':
+                    # MSVC refuses to construct an array of zero size, so
+                    # the test only succeeds when i is sizeof(element) + 1
+                    return i - 1
                 return i
         raise EnvironmentException('Cross checking offsetof overflowed.')
 
@@ -653,11 +700,19 @@ int main(int argc, char **argv) {
         return align
 
     def has_function(self, funcname, prefix, env, extra_args=[]):
+        """
+        First, this function looks for the symbol in the default libraries
+        provided by the compiler (stdlib + a few others usually). If that
+        fails, it checks if any of the headers specified in the prefix provide
+        an implementation of the function, and if that fails, it checks if it's
+        implemented as a compiler-builtin.
+        """
         # Define the symbol to something else in case it is defined by the
         # includes or defines listed by the user `{0}` or by the compiler.
         # Then, undef the symbol to get rid of it completely.
         templ = '''
         #define {1} meson_disable_define_of_{1}
+        #include <limits.h>
         {0}
         #undef {1}
         '''
@@ -675,11 +730,14 @@ int main(int argc, char **argv) {
         # glibc defines functions that are not available on Linux as stubs that
         # fail with ENOSYS (such as e.g. lchmod). In this case we want to fail
         # instead of detecting the stub as a valid symbol.
-        templ += '''
+        # We always include limits.h above to ensure that these are defined for
+        # stub functions.
+        stubs_fail = '''
         #if defined __stub_{1} || defined __stub___{1}
         fail fail fail this function is not going to work
         #endif
         '''
+        templ += stubs_fail
 
         # And finally the actual function call
         templ += '''
@@ -698,12 +756,19 @@ int main(int argc, char **argv) {
                 raise EnvironmentException('Cross variable {0} is not a boolean.'.format(varname))
         if self.links(templ.format(prefix, funcname), extra_args):
             return True
+        # Add -O0 to ensure that the symbol isn't optimized away by the compiler
+        extra_args += self.get_no_optimization_args()
+        # Sometimes the implementation is provided by the header, or the header
+        # redefines the symbol to be something else. In that case, we want to
+        # still detect the function. We still want to fail if __stub_foo or
+        # _stub_foo are defined, of course.
+        header_templ = '#include <limits.h>\n{0}\n' + stubs_fail + '\nint main() {{ {1}; }}'
+        if self.links(header_templ.format(prefix, funcname), extra_args):
+            return True
         # Some functions like alloca() are defined as compiler built-ins which
         # are inlined by the compiler, so test for that instead. Built-ins are
         # special functions that ignore all includes and defines, so we just
         # directly try to link via main().
-        # Add -O0 to ensure that the symbol isn't optimized away by the compiler
-        extra_args += self.get_no_optimization_args()
         return self.links('int main() {{ {0}; }}'.format('__builtin_' + funcname), extra_args)
 
     def has_member(self, typename, membername, prefix, extra_args=[]):
@@ -767,42 +832,8 @@ class CPPCompiler(CCompiler):
         return False
 
     def sanity_check(self, work_dir):
-        source_name = os.path.join(work_dir, 'sanitycheckcpp.cc')
-        binary_name = os.path.join(work_dir, 'sanitycheckcpp')
-        ofile = open(source_name, 'w')
-        ofile.write('class breakCCompiler;int main(int argc, char **argv) { return 0; }\n')
-        ofile.close()
-        if self.is_cross and self.exe_wrapper is None:
-            # Skipping link because of the same reason as for C.
-            # The comment in CCompiler explains why this is done.
-            extra_flags = ['-c']
-        else:
-            extra_flags = []
-        cmdlist = self.exelist + extra_flags + [source_name, '-o', binary_name]
-        pc = subprocess.Popen(cmdlist, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        (stdo, stde) = pc.communicate()
-        stdo = stdo.decode()
-        stde = stde.decode()
-        mlog.debug('Sanity check compiler command line:', ' '.join(cmdlist))
-        mlog.debug('Sanity check compile stdout:')
-        mlog.debug(stdo)
-        mlog.debug('-----\nSanity check compile stderr:')
-        mlog.debug(stde)
-        mlog.debug('-----')
-        pc.wait()
-        if pc.returncode != 0:
-            raise EnvironmentException('Compiler %s can not compile programs.' % self.name_string())
-        if self.is_cross:
-            if self.exe_wrapper is None:
-                # Can't check if the binaries run so we have to assume they do
-                return
-            cmdlist = self.exe_wrapper + [binary_name]
-        else:
-            cmdlist = [binary_name]
-        pe = subprocess.Popen(cmdlist)
-        pe.wait()
-        if pe.returncode != 0:
-            raise EnvironmentException('Executables created by C++ compiler %s are not runnable.' % self.name_string())
+        code = 'class breakCCompiler;int main(int argc, char **argv) { return 0; }\n'
+        return self.sanity_check_impl(work_dir, 'sanitycheckcpp.cc', code)
 
 class ObjCCompiler(CCompiler):
     def __init__(self, exelist, version, is_cross, exe_wrap):
@@ -1345,40 +1376,9 @@ class VisualStudioCCompiler(CCompiler):
         objname = os.path.splitext(pchname)[0] + '.obj'
         return (objname, ['/Yc' + header, '/Fp' + pchname, '/Fo' + objname ])
 
-    def sanity_check(self, work_dir):
-        source_name = 'sanitycheckc.c'
-        binary_name = 'sanitycheckc'
-        ofile = open(os.path.join(work_dir, source_name), 'w')
-        ofile.write('int main(int argc, char **argv) { return 0; }\n')
-        ofile.close()
-        pc = subprocess.Popen(self.exelist + [source_name, '/Fe' + binary_name],
-                              stdout=subprocess.DEVNULL,
-                              stderr=subprocess.DEVNULL,
-                              cwd=work_dir)
-        pc.wait()
-        if pc.returncode != 0:
-            raise EnvironmentException('Compiler %s can not compile programs.' % self.name_string())
-        pe = subprocess.Popen(os.path.join(work_dir, binary_name))
-        pe.wait()
-        if pe.returncode != 0:
-            raise EnvironmentException('Executables created by C++ compiler %s are not runnable.' % self.name_string())
-
     def build_rpath_args(self, build_dir, rpath_paths, install_rpath):
         return []
 
-    def find_library(self, libname, extra_dirs):
-        code = '''int main(int argc, char **argv) {
-    return 0;
-}
-        '''
-        args = []
-        for i in extra_dirs:
-            args += self.get_linker_search_args(i)
-        args.append(libname + '.lib')
-        if self.links(code, extra_args=args):
-            return args
-        return None
-
     # FIXME, no idea what these should be.
     def thread_flags(self):
         return []
@@ -1440,24 +1440,6 @@ class VisualStudioCPPCompiler(VisualStudioCCompiler):
             return True
         return False
 
-    def sanity_check(self, work_dir):
-        source_name = 'sanitycheckcpp.cpp'
-        binary_name = 'sanitycheckcpp'
-        ofile = open(os.path.join(work_dir, source_name), 'w')
-        ofile.write('class BreakPlainC;int main(int argc, char **argv) { return 0; }\n')
-        ofile.close()
-        pc = subprocess.Popen(self.exelist + [source_name, '/Fe' + binary_name],
-                              stdout=subprocess.DEVNULL,
-                              stderr=subprocess.DEVNULL,
-                              cwd=work_dir)
-        pc.wait()
-        if pc.returncode != 0:
-            raise EnvironmentException('Compiler %s can not compile programs.' % self.name_string())
-        pe = subprocess.Popen(os.path.join(work_dir, binary_name))
-        pe.wait()
-        if pe.returncode != 0:
-            raise EnvironmentException('Executables created by C++ compiler %s are not runnable.' % self.name_string())
-
     def get_options(self):
         return {'cpp_eh' : coredata.UserComboOption('cpp_eh',
                                                     'C++ exception handling type.',
@@ -1713,8 +1695,11 @@ class GnuCPPCompiler(CPPCompiler):
 
     def get_options(self):
         opts = {'cpp_std' : coredata.UserComboOption('cpp_std', 'C++ language standard to use',
-                                                     ['none', 'c++03', 'c++11', 'c++14'],
-                                                     'none')}
+                                                     ['none', 'c++03', 'c++11', 'c++14', 'c++1z'],
+                                                     'none'),
+                'cpp_debugstl': coredata.UserBooleanOption('cpp_debugstl',
+                                                           'STL debug mode',
+                                                           False)}
         if self.gcc_type == GCC_MINGW:
             opts.update({
                 'cpp_winlibs': coredata.UserStringArrayOption('c_winlibs', 'Standard Win libraries to link against',
@@ -1727,6 +1712,8 @@ class GnuCPPCompiler(CPPCompiler):
         std = options['cpp_std']
         if std.value != 'none':
             args.append('-std=' + std.value)
+        if options['cpp_debugstl'].value:
+            args.append('-D_GLIBCXX_DEBUG=1')
         return args
 
     def get_option_link_args(self, options):
@@ -1763,7 +1750,7 @@ class ClangCPPCompiler(CPPCompiler):
 
     def get_options(self):
         return {'cpp_std' : coredata.UserComboOption('cpp_std', 'C++ language standard to use',
-                                                   ['none', 'c++03', 'c++11', 'c++14'],
+                                                   ['none', 'c++03', 'c++11', 'c++14', 'c++1z'],
                                                    'none')}
 
     def get_option_compile_args(self, options):
