@@ -59,8 +59,9 @@ class Dependency():
         return False
 
 class InternalDependency(Dependency):
-    def __init__(self, incdirs, compile_args, link_args, libraries, sources, ext_deps):
+    def __init__(self, version, incdirs, compile_args, link_args, libraries, sources, ext_deps):
         super().__init__()
+        self.version = version
         self.include_directories = incdirs
         self.compile_args = compile_args
         self.link_args = link_args
@@ -74,6 +75,9 @@ class InternalDependency(Dependency):
     def get_link_args(self):
         return self.link_args
 
+    def get_version(self):
+        return self.version
+
 class PkgConfigDependency(Dependency):
     pkgconfig_found = None
 
@@ -188,6 +192,9 @@ class PkgConfigDependency(Dependency):
     def get_modversion(self):
         return self.modversion
 
+    def get_version(self):
+        return self.get_modversion()
+
     def get_compile_args(self):
         return self.cargs
 
@@ -1141,7 +1148,15 @@ def get_dep_identifier(name, kwargs):
         modlist = [modlist]
     for module in modlist:
         elements.append(module)
-    return '/'.join(elements) + '/main' + str(kwargs.get('main', False)) + '/static' + str(kwargs.get('static', False))
+    # We use a tuple because we need a non-mutable structure to use as the key
+    # of a dictionary and a string has potential for name collisions
+    identifier = tuple(elements)
+    identifier += ('main', kwargs.get('main', False))
+    identifier += ('static', kwargs.get('static', False))
+    if 'fallback' in kwargs:
+        f = kwargs.get('fallback')
+        identifier += ('fallback', f[0], f[1])
+    return identifier
 
 def find_external_dependency(name, environment, kwargs):
     required = kwargs.get('required', True)
