@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import os, re, subprocess
+import os, re, subprocess, platform
 from . import coredata, mesonlib
 from .compilers import *
 import configparser
@@ -52,6 +52,33 @@ def detect_ninja():
         if p.returncode == 0:
             return n
 
+def detect_cpu_family():
+    """
+    Python is inconsistent in its platform module.
+    It returns different values for the same cpu.
+    For x86 it might return 'x86', 'i686' or somesuch.
+    Do some canonicalization.
+    """
+    trial = platform.machine().lower()
+    if trial.startswith('i') and trial.endswith('86'):
+        return 'x86'
+    if trial.startswith('arm'):
+        return 'arm'
+    if trial == 'amd64':
+        return 'x86_64'
+    # Add fixes here as bugs are reported.
+    return trial
+
+def detect_cpu():
+    trial = platform.machine().lower()
+    if trial == 'amd64':
+        return 'x86_64'
+    # Add fixes here as bugs are reported.
+    return trial
+
+def detect_system():
+    return platform.system().lower()
+
 
 class Environment():
     private_dir = 'meson-private'
@@ -131,7 +158,8 @@ class Environment():
         coredata.save(self.coredata, cdf)
 
     def get_script_dir(self):
-        return os.path.join(os.path.dirname(self.meson_script_file), '../scripts')
+        import mesonbuild.scripts
+        return os.path.dirname(mesonbuild.scripts.__file__)
 
     def get_log_dir(self):
         return self.log_dir
@@ -181,7 +209,10 @@ class Environment():
             compilers = [self.cross_info.config['binaries']['c']]
             ccache = []
             is_cross = True
-            exe_wrap = self.cross_info.config['binaries'].get('exe_wrapper', None)
+            if self.cross_info.need_exe_wrapper():
+                exe_wrap = self.cross_info.config['binaries'].get('exe_wrapper', None)
+            else:
+                exe_wrap = []
         elif evar in os.environ:
             compilers = os.environ[evar].split()
             ccache = []
@@ -192,6 +223,7 @@ class Environment():
             ccache = self.detect_ccache()
             is_cross = False
             exe_wrap = None
+        popen_exceptions = {}
         for compiler in compilers:
             try:
                 basename = os.path.basename(compiler).lower()
@@ -199,9 +231,10 @@ class Environment():
                     arg = '/?'
                 else:
                     arg = '--version'
-                p = subprocess.Popen([compiler] + [arg], stdout=subprocess.PIPE,
+                p = subprocess.Popen([compiler, arg], stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE)
-            except OSError:
+            except OSError as e:
+                popen_exceptions[' '.join([compiler, arg])] = e
                 continue
             (out, err) = p.communicate()
             out = out.decode(errors='ignore')
@@ -232,14 +265,22 @@ class Environment():
                 # everything else to stdout. Why? Lord only knows.
                 version = re.search(Environment.version_regex, err).group()
                 return VisualStudioCCompiler([compiler], version, is_cross, exe_wrap)
-        raise EnvironmentException('Unknown compiler(s): "' + ', '.join(compilers) + '"')
+        errmsg = 'Unknown compiler(s): "' + ', '.join(compilers) + '"'
+        if popen_exceptions:
+            errmsg += '\nThe follow exceptions were encountered:'
+            for (c, e) in popen_exceptions.items():
+                errmsg += '\nRunning "{0}" gave "{1}"'.format(c, e)
+        raise EnvironmentException(errmsg)
 
     def detect_fortran_compiler(self, want_cross):
         evar = 'FC'
         if self.is_cross_build() and want_cross:
             compilers = [self.cross_info['fortran']]
             is_cross = True
-            exe_wrap = self.cross_info.get('exe_wrapper', None)
+            if self.cross_info.need_exe_wrapper():
+                exe_wrap = self.cross_info.get('exe_wrapper', None)
+            else:
+                exe_wrap = []
         elif evar in os.environ:
             compilers = os.environ[evar].split()
             is_cross = False
@@ -248,13 +289,15 @@ class Environment():
             compilers = self.default_fortran
             is_cross = False
             exe_wrap = None
+        popen_exceptions = {}
         for compiler in compilers:
             for arg in ['--version', '-V']:
                 try:
-                    p = subprocess.Popen([compiler] + [arg],
+                    p = subprocess.Popen([compiler, arg],
                                          stdout=subprocess.PIPE,
                                          stderr=subprocess.PIPE)
-                except OSError:
+                except OSError as e:
+                    popen_exceptions[' '.join([compiler, arg])] = e
                     continue
                 (out, err) = p.communicate()
                 out = out.decode(errors='ignore')
@@ -266,7 +309,13 @@ class Environment():
                     version = vmatch.group(0)
 
                 if 'GNU Fortran' in out:
-                    return GnuFortranCompiler([compiler], version, GCC_STANDARD, is_cross, exe_wrap)
+                    if mesonlib.is_osx():
+                        gcctype = GCC_OSX
+                    elif mesonlib.is_windows():
+                        gcctype = GCC_MINGW
+                    else:
+                        gcctype = GCC_STANDARD
+                    return GnuFortranCompiler([compiler], version, gcctype, is_cross, exe_wrap)
 
                 if 'G95' in out:
                     return G95FortranCompiler([compiler], version, is_cross, exe_wrap)
@@ -292,8 +341,12 @@ class Environment():
 
                 if 'NAG Fortran' in err:
                     return NAGFortranCompiler([compiler], version, is_cross, exe_wrap)
-
-        raise EnvironmentException('Unknown compiler(s): "' + ', '.join(compilers) + '"')
+        errmsg = 'Unknown compiler(s): "' + ', '.join(compilers) + '"'
+        if popen_exceptions:
+            errmsg += '\nThe follow exceptions were encountered:'
+            for (c, e) in popen_exceptions.items():
+                errmsg += '\nRunning "{0}" gave "{1}"'.format(c, e)
+        raise EnvironmentException(errmsg)
 
     def get_scratch_dir(self):
         return self.scratch_dir
@@ -308,7 +361,10 @@ class Environment():
             compilers = [self.cross_info.config['binaries']['cpp']]
             ccache = []
             is_cross = True
-            exe_wrap = self.cross_info.config['binaries'].get('exe_wrapper', None)
+            if self.cross_info.need_exe_wrapper():
+                exe_wrap = self.cross_info.config['binaries'].get('exe_wrapper', None)
+            else:
+                exe_wrap = []
         elif evar in os.environ:
             compilers = os.environ[evar].split()
             ccache = []
@@ -319,6 +375,7 @@ class Environment():
             ccache = self.detect_ccache()
             is_cross = False
             exe_wrap = None
+        popen_exceptions = {}
         for compiler in compilers:
             basename = os.path.basename(compiler).lower()
             if basename == 'cl' or basename == 'cl.exe':
@@ -329,7 +386,8 @@ class Environment():
                 p = subprocess.Popen([compiler, arg],
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE)
-            except OSError:
+            except OSError as e:
+                popen_exceptions[' '.join([compiler, arg])] = e
                 continue
             (out, err) = p.communicate()
             out = out.decode(errors='ignore')
@@ -358,13 +416,21 @@ class Environment():
             if 'Microsoft' in out or 'Microsoft' in err:
                 version = re.search(Environment.version_regex, err).group()
                 return VisualStudioCPPCompiler([compiler], version, is_cross, exe_wrap)
-        raise EnvironmentException('Unknown compiler(s) "' + ', '.join(compilers) + '"')
+        errmsg = 'Unknown compiler(s): "' + ', '.join(compilers) + '"'
+        if popen_exceptions:
+            errmsg += '\nThe follow exceptions were encountered:'
+            for (c, e) in popen_exceptions.items():
+                errmsg += '\nRunning "{0}" gave "{1}"'.format(c, e)
+        raise EnvironmentException(errmsg)
 
     def detect_objc_compiler(self, want_cross):
         if self.is_cross_build() and want_cross:
             exelist = [self.cross_info['objc']]
             is_cross = True
-            exe_wrap = self.cross_info.get('exe_wrapper', None)
+            if self.cross_info.need_exe_wrapper():
+                exe_wrap = self.cross_info.get('exe_wrapper', None)
+            else:
+                exe_wrap = []
         else:
             exelist = self.get_objc_compiler_exelist()
             is_cross = False
@@ -394,7 +460,10 @@ class Environment():
         if self.is_cross_build() and want_cross:
             exelist = [self.cross_info['objcpp']]
             is_cross = True
-            exe_wrap = self.cross_info.get('exe_wrapper', None)
+            if self.cross_info.need_exe_wrapper():
+                exe_wrap = self.cross_info.get('exe_wrapper', None)
+            else:
+                exe_wrap = []
         else:
             exelist = self.get_objcpp_compiler_exelist()
             is_cross = False
@@ -696,7 +765,22 @@ class CrossBuildInfo():
     def has_target(self):
         return 'target_machine' in self.config
 
+    def has_stdlib(self, language):
+        return language + '_stdlib' in self.config['properties']
+
+    def get_stdlib(self, language):
+        return self.config['properties'][language + '_stdlib']
+
     # Wehn compiling a cross compiler we use the native compiler for everything.
     # But not when cross compiling a cross compiler.
     def need_cross_compiler(self):
         return 'host_machine' in self.config
+
+    def need_exe_wrapper(self):
+        if self.has_host() and detect_cpu_family() == 'x86_64' and \
+           self.config['host_machine']['cpu_family'] == 'x86' and \
+           self.config['host_machine']['system'] == detect_system():
+            # Can almost always run 32-bit binaries on 64-bit natively if the
+            # host and build systems are the same
+            return False
+        return True
