@@ -132,7 +132,7 @@ numpart = re.compile('[0-9.]+')
 def version_compare(vstr1, vstr2):
     match = numpart.match(vstr1.strip())
     if match is None:
-        raise MesonException('Unconparable version string %s.' % vstr1)
+        raise MesonException('Uncomparable version string %s.' % vstr1)
     vstr1 = match.group(0)
     if vstr2.startswith('>='):
         cmpop = operator.ge
@@ -252,7 +252,9 @@ def do_mesondefine(line, confdata):
 
 def do_conf_file(src, dst, confdata):
     data = open(src).readlines()
-    regex = re.compile('@(.*?)@')
+    # Only allow (a-z, A-Z, 0-9, _, -) as valid characters for a define
+    # Also allow escaping '@' with '\@'
+    regex = re.compile(r'[^\\]?@([-a-zA-Z0-9_]+)@')
     result = []
     for line in data:
         if line.startswith('#mesondefine'):
@@ -265,6 +267,27 @@ def do_conf_file(src, dst, confdata):
     shutil.copymode(src, dst_tmp)
     replace_if_different(dst, dst_tmp)
 
+def dump_conf_header(ofilename, cdata):
+    with open(ofilename, 'w') as ofile:
+        ofile.write('''/*
+ * Autogenerated by the Meson build system.
+ * Do not edit, your changes will be lost.
+ */
+
+#pragma once
+
+''')
+        for k in sorted(cdata.keys()):
+            v = cdata.get(k)
+            if isinstance(v, bool):
+                if v:
+                    ofile.write('#define %s\n\n' % k)
+                else:
+                    ofile.write('#undef %s\n\n' % k)
+            elif isinstance(v, (int, str)):
+                ofile.write('#define %s %s\n\n' % (k, v))
+            else:
+                raise MesonException('Unknown data type in configuration file entry: ' + k)
 
 def replace_if_different(dst, dst_tmp):
     # If contents are identical, don't touch the file to prevent
