@@ -23,7 +23,6 @@ import platform
 from . import mlog, coredata
 from .mesonlib import MesonException
 
-backendlist = ['ninja', 'vs2010', 'xcode']
 
 parser = argparse.ArgumentParser()
 
@@ -139,6 +138,9 @@ itself as required.'''
         elif self.options.backend == 'vs2010':
             from .backend import vs2010backend
             g = vs2010backend.Vs2010Backend(b)
+        elif self.options.backend == 'vs2015':
+            from .backend import vs2015backend
+            g = vs2015backend.Vs2015Backend(b)
         elif self.options.backend == 'xcode':
             from .backend import xcodebackend
             g = xcodebackend.XCodeBackend(b)
@@ -163,7 +165,10 @@ itself as required.'''
 def run_script_command(args):
     cmdname = args[0]
     cmdargs = args[1:]
-    if cmdname == 'test':
+    if cmdname == 'exe':
+        import mesonbuild.scripts.meson_exe as abc
+        cmdfunc = abc.run
+    elif cmdname == 'test':
         import mesonbuild.scripts.meson_test as abc
         cmdfunc = abc.run
     elif cmdname == 'benchmark':
@@ -193,9 +198,15 @@ def run_script_command(args):
     elif cmdname == 'symbolextractor':
         import mesonbuild.scripts.symbolextractor as abc
         cmdfunc = abc.run
+    elif cmdname == 'scanbuild':
+        import mesonbuild.scripts.scanbuild as abc
+        cmdfunc = abc.run
     elif cmdname == 'vcstagger':
         import mesonbuild.scripts.vcstagger as abc
         cmdfunc = abc.run
+    elif cmdname == 'gettext':
+        import mesonbuild.scripts.gettext as abc
+        cmdfunc = abc.run
     else:
         raise MesonException('Unknown internal command {}.'.format(cmdname))
     return cmdfunc(cmdargs)
