@@ -289,11 +289,21 @@ class GnomeModule:
                 '--modulename=' + modulename]
         args += self.unpack_args('--htmlargs=', 'html_args', kwargs)
         args += self.unpack_args('--scanargs=', 'scan_args', kwargs)
+        args += self.unpack_args('--fixxrefargs=', 'fixxref_args', kwargs)
         res = [build.RunTarget(targetname, command[0], command[1:] + args, state.subdir)]
         if kwargs.get('install', True):
             res.append(build.InstallScript(command + args))
         return res
 
+    def gtkdoc_html_dir(self, state, args, kwarga):
+        if len(args) != 1:
+            raise MesonException('Must have exactly one argument.')
+        modulename = args[0]
+        if not isinstance(modulename, str):
+            raise MesonException('Argument must be a string')
+        return os.path.join('share/gtkdoc/html', modulename)
+
+
     def unpack_args(self, arg, kwarg_name, kwargs):
         try:
             new_args = kwargs[kwarg_name]
