@@ -18,6 +18,22 @@ import sys, pickle, os, shutil, subprocess, gzip, platform
 from glob import glob
 from mesonbuild.scripts import depfixer
 
+def do_copy(from_file, to_file):
+    try:
+        # Python's copyfile fails if the target file already exists.
+        os.unlink(to_file)
+    except FileNotFoundError:
+        pass
+    shutil.copyfile(from_file, to_file)
+    shutil.copystat(from_file, to_file)
+
+def destdir_join(d1, d2):
+    # c:\destdir + c:\prefix must produce c:\destdir\prefix
+    if len(d1) > 1 and d1[1] == ':' and \
+        len(d2) > 1 and d2[1] == ':':
+        return d1 + d2[2:]
+    return d1 + d2
+
 def do_install(datafilename):
     ifile = open(datafilename, 'rb')
     d = pickle.load(ifile)
@@ -26,58 +42,66 @@ def do_install(datafilename):
         d.destdir = os.environ[destdir_var]
     else:
         d.destdir = ''
-    d.fullprefix = d.destdir + d.prefix
+    d.fullprefix = destdir_join(d.destdir, d.prefix)
 
     install_subdirs(d) # Must be first, because it needs to delete the old subtree.
     install_targets(d)
     install_headers(d)
     install_man(d)
     install_data(d)
-    install_po(d)
     run_install_script(d)
 
-def install_subdirs(d):
-    for (src_dir, dst_dir) in d.install_subdirs:
+def install_subdirs(data):
+    for (src_dir, inst_dir, dst_dir) in data.install_subdirs:
+        if src_dir.endswith('/'):
+            src_dir = src_dir[:-1]
+        src_prefix = os.path.join(src_dir, inst_dir)
+        print('Installing subdir %s to %s.' % (src_prefix, dst_dir))
         if os.path.isabs(dst_dir):
-            dst_dir = d.destdir + dst_dir
+            dst_dir = destdir_join(data.destdir, dst_dir)
         else:
-            dst_dir = d.fullprefix + dst_dir
-        # Python's copytree works in strange ways.
-        last_level = os.path.split(src_dir)[-1]
-        final_dst = os.path.join(dst_dir, last_level)
-# Don't do rmtree because final_dst might point to e.g. /var/www
-# We might need to revert to walking the directory tree by hand.
-#        shutil.rmtree(final_dst, ignore_errors=True)
-        shutil.copytree(src_dir, final_dst, symlinks=True)
-        print('Installing subdir %s to %s.' % (src_dir, dst_dir))
-
-def install_po(d):
-    packagename = d.po_package_name
-    for f in d.po:
-        srcfile = f[0]
-        localedir = f[1]
-        languagename = f[2]
-        outfile = os.path.join(d.fullprefix, localedir, languagename, 'LC_MESSAGES',
-                               packagename + '.mo')
-        os.makedirs(os.path.split(outfile)[0], exist_ok=True)
-        shutil.copyfile(srcfile, outfile)
-        shutil.copystat(srcfile, outfile)
-        print('Installing %s to %s.' % (srcfile, outfile))
+            dst_dir = data.fullprefix + dst_dir
+        if not os.path.exists(dst_dir):
+            os.makedirs(dst_dir)
+        for root, dirs, files in os.walk(src_prefix):
+            for d in dirs:
+                abs_src = os.path.join(src_dir, root, d)
+                filepart = abs_src[len(src_dir)+1:]
+                abs_dst = os.path.join(dst_dir, filepart)
+                if os.path.isdir(abs_dst):
+                    continue
+                if os.path.exists(abs_dst):
+                    print('Tried to copy directory %s but a file of that name already exists.' % abs_dst)
+                    sys.exit(1)
+                os.makedirs(abs_dst)
+                shutil.copystat(abs_src, abs_dst)
+            for f in files:
+                abs_src = os.path.join(src_dir, root, f)
+                filepart = abs_src[len(src_dir)+1:]
+                abs_dst = os.path.join(dst_dir, filepart)
+                if os.path.isdir(abs_dst):
+                    print('Tried to copy file %s but a directory of that name already exists.' % abs_dst)
+                if os.path.exists(abs_dst):
+                    os.unlink(abs_dst)
+                parent_dir = os.path.split(abs_dst)[0]
+                if not os.path.isdir(parent_dir):
+                    os.mkdir(parent_dir)
+                    shutil.copystat(os.path.split(abs_src)[0], parent_dir)
+                shutil.copy2(abs_src, abs_dst, follow_symlinks=False)
 
 def install_data(d):
     for i in d.data:
         fullfilename = i[0]
         outfilename = i[1]
         if os.path.isabs(outfilename):
-            outdir = d.destdir + os.path.split(outfilename)[0]
-            outfilename = d.destdir + outfilename
+            outdir = destdir_join(d.destdir, os.path.split(outfilename)[0])
+            outfilename = destdir_join(d.destdir, outfilename)
         else:
             outdir = os.path.join(d.fullprefix, os.path.split(outfilename)[0])
             outfilename = os.path.join(outdir, os.path.split(outfilename)[1])
         os.makedirs(outdir, exist_ok=True)
         print('Installing %s to %s.' % (fullfilename, outdir))
-        shutil.copyfile(fullfilename, outfilename)
-        shutil.copystat(fullfilename, outfilename)
+        do_copy(fullfilename, outfilename)
 
 def install_man(d):
     for m in d.man:
@@ -89,9 +113,9 @@ def install_man(d):
         print('Installing %s to %s.' % (full_source_filename, outdir))
         if outfilename.endswith('.gz') and not full_source_filename.endswith('.gz'):
             open(outfilename, 'wb').write(gzip.compress(open(full_source_filename, 'rb').read()))
+            shutil.copystat(full_source_filename, outfilename)
         else:
-            shutil.copyfile(full_source_filename, outfilename)
-        shutil.copystat(full_source_filename, outfilename)
+            do_copy(full_source_filename, outfilename)
 
 def install_headers(d):
     for t in d.headers:
@@ -101,8 +125,7 @@ def install_headers(d):
         outfilename = os.path.join(outdir, fname)
         print('Installing %s to %s' % (fname, outdir))
         os.makedirs(outdir, exist_ok=True)
-        shutil.copyfile(fullfilename, outfilename)
-        shutil.copystat(fullfilename, outfilename)
+        do_copy(fullfilename, outfilename)
 
 def run_install_script(d):
     env = {'MESON_SOURCE_ROOT' : d.source_dir,
@@ -113,11 +136,12 @@ def run_install_script(d):
     child_env.update(env)
 
     for i in d.install_scripts:
+        final_command = i.cmd_arr
         script = i.cmd_arr[0]
         print('Running custom install script %s' % script)
         suffix = os.path.splitext(script)[1].lower()
         if platform.system().lower() == 'windows' and suffix != '.bat':
-            first_line = open(script).readline().strip()
+            first_line = open(script, encoding='latin_1', errors='ignore').readline().strip()
             if first_line.startswith('#!'):
                 if shutil.which(first_line[2:]):
                     commands = [first_line[2:]]
@@ -125,11 +149,8 @@ def run_install_script(d):
                     commands = first_line[2:].split('#')[0].strip().split()
                     commands[0] = shutil.which(commands[0].split('/')[-1])
                     if commands[0] is None:
-                        commands
                         raise RuntimeError("Don't know how to run script %s." % script)
                 final_command = commands + [script] + i.cmd_arr[1:]
-        else:
-            final_command = i.cmd_arr
         try:
             rc = subprocess.call(final_command, env=child_env)
             if rc != 0:
@@ -179,8 +200,7 @@ def install_targets(d):
         install_rpath = t[4]
         print('Installing %s to %s' % (fname, outname))
         os.makedirs(outdir, exist_ok=True)
-        shutil.copyfile(fname, outname)
-        shutil.copystat(fname, outname)
+        do_copy(fname, outname)
         if should_strip:
             print('Stripping target')
             ps = subprocess.Popen(['strip', outname], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
